// This file was generated by the Gtk# code generator.
// Any changes made will be lost if regenerated.

namespace GtkBeans {

	using System;

	public delegate void BuilderConnectFunc(GtkBeans.Builder builder, GLib.Object objekt, string signal_name, string handler_name, GLib.Object connect_object, GLib.ConnectFlags flags);

}
