/*
 * Copyright (C) 2010 Jordi Mas i Hernàndez <jmas@softcatala.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System; 

using gbrainy.Core.Main;

namespace gbrainy.Core.Toolkit
{
	public class DrawEventArgs: EventArgs
	{
		public DrawEventArgs (CairoContextEx gr, double width, double height, bool rtl, object data)
		{
			Context = gr;
			Width = width;
			Height = height;
			Rtl = rtl;
			Data = data;
		}

		public CairoContextEx Context { get; set; }
		public double Width { get; set; }
		public double Height { get; set; }
		public bool Rtl { get; set; }
		public object Data { get; set; }
	}
}
